from dcard.base_classes import ResponseExt, HandlerExt


class Response(ResponseExt):
    pass


class Handler(HandlerExt):
    endpoint = '/card/print/{card_id}/{template_id}'

    def __call__(self, public_id):
        url = self.get_url(card_id=public_id, template_id=self.config['TEMPLATE_ID'])
        response = self.get(url)
        return Response(response).data



from schema import Schema

from dcard.base_classes import ResponseExt, HandlerExt


class Response(ResponseExt):
    pass


class Handler(HandlerExt):
    endpoint = '/card/push/{public_id}'
    schema = Schema({
        'city_id': int
    })

    def __call__(self, public_id, data):
        url = self.get_url(public_id=public_id)
        payload = self.schema.validate(data)
        response = self.post(url, payload)
        return Response(response).data



from datetime import date
from enum import IntEnum

from schema import Schema, And, Use, Optional

from dcard.base_classes import ResponseExt, HandlerExt


class OwnerType(IntEnum):
    INDIVIDUAL = 1
    ENTITY = 2


class DocumentType(IntEnum):
    STS = 1
    PTS = 2


class Category(IntEnum):
    A = 1
    B = 2
    C = 3
    D = 4
    E = 6


class CustomsCategory(IntEnum):
    M1 = 2
    N1 = 3


class FuelType(IntEnum):
    PETROL = 1
    DIESEL = 2


class BrakeType(IntEnum):
    HYDRAULIC = 1
    PNEUMATIC = 2
    COMBINED = 3
    MECHANICAL = 4


class Response(ResponseExt):
    pass


class Handler(HandlerExt):
    endpoint = '/card'
    schema = Schema({
        'owner_type': And(OwnerType, Use(int)),
        Optional('owner_firm'): str,
        'owner_name': str,
        'owner_surname': str,
        Optional('owner_patronymic'): str,
        'document_type': And(DocumentType, Use(int)),
        'document_series': str,
        'document_number': str,
        'document_issued_date': And(date, Use(lambda x: x.strftime('%d.%m.%Y'))),
        'document_issued_by': str,
        'ts_category': And(Category, Use(int)),
        Optional('rc'): And(CustomsCategory, Use(int)),
        Optional('ts_reg_number'): str,
        Optional('ts_vin'): str,
        Optional('ts_body'): str,
        Optional('ts_frame'): str,
        'ts_mark': str,
        'ts_model': str,
        'ts_year': int,
        'ts_mass_base': int,
        'ts_mass_max': int,
        Optional('ts_mileage'): int,
        Optional('ts_fuel_type'): And(FuelType, Use(int)),
        'ts_brakes_type': And(BrakeType, Use(int)),
        'tires_id': int,
        'ts_taxi': And(bool, Use(int)),
        'ts_training': And(bool, Use(int)),
        'ts_dangerous': And(bool, Use(int)),
    })

    def __call__(self, data):
        url = self.get_url()
        payload = self.schema.validate(data)
        response = self.post(url, payload)
        return Response(response).data



from dcard.base_classes import HandlerExt, Response as BaseResponse
from io import BytesIO


class Response(BaseResponse):
    def __init__(self, response):
        content = response.content
        self._data = BytesIO(content)


class Handler(HandlerExt):
    def __call__(self, url):
        response = self.get(url)
        return Response(response).data



from enum import Enum, auto as a
from types import MappingProxyType

from requests import Session

from dcard.actions import CreateCardRequest, RetrieveCardRequest, ListCities, ListTires, PushCard, UpdateCard, DenyCard, \
    Print, Download


class RequestStrategy:
    def __init__(self, timeout):
        self.timeout = timeout
        self.session = Session()

    def get(self, url, **kwargs):
        return self.session.get(url, **self._get_kwargs(kwargs))

    def post(self, url, **kwargs):
        return self.session.post(url, **self._get_kwargs(kwargs))

    def _get_kwargs(self, kwargs):
        return {'timeout': self.timeout, **kwargs}


class Handlers(Enum):
    CREATE_CARD_REQUEST = a()
    RETRIEVE_CARD_REQUEST = a()
    LIST_CITIES = a()
    LIST_TIRES = a()
    PUSH_CARD = a()
    UPDATE_CARD = a()
    DENY_CARD = a()
    PRINT = a()
    DOWNLOAD = a()


HANDLERS_MAP = {
    Handlers.CREATE_CARD_REQUEST: CreateCardRequest,
    Handlers.RETRIEVE_CARD_REQUEST: RetrieveCardRequest,
    Handlers.LIST_CITIES: ListCities,
    Handlers.LIST_TIRES: ListTires,
    Handlers.PUSH_CARD: PushCard,
    Handlers.UPDATE_CARD: UpdateCard,
    Handlers.DENY_CARD: DenyCard,
    Handlers.PRINT: Print,
    Handlers.DOWNLOAD: Download,
}


class APIWrapper:
    BASE_SETTINGS = MappingProxyType({
        'BASE_URL': 'https://www.dcard.ws/api',
        'TIMEOUT': 60,
        'TEMPLATE_ID': 3
    })

    def __init__(self, config_dict):
        config_dict = MappingProxyType(
            {**self.BASE_SETTINGS, **config_dict}
        )
        request_strategy = RequestStrategy(config_dict['TIMEOUT'])

        self._handlers = {
            enum_key: handler_class(config_dict, request_strategy)
            for enum_key, handler_class in HANDLERS_MAP.items()
        }

    def create_card_request(self, data):
        handler = self._handlers[Handlers.CREATE_CARD_REQUEST]
        return handler(data)

    def retrieve_card_request(self, public_id):
        handler = self._handlers[Handlers.RETRIEVE_CARD_REQUEST]
        return handler(public_id)

    def push_card(self, public_id, data):
        handler = self._handlers[Handlers.PUSH_CARD]
        return handler(public_id, data)

    def deny(self, public_id):
        handler = self._handlers[Handlers.DENY_CARD]
        return handler(public_id)

    def list_cities(self):
        handler = self._handlers[Handlers.LIST_CITIES]
        return handler()

    def list_tires(self):
        handler = self._handlers[Handlers.LIST_TIRES]
        return handler()

    def print(self, public_id):
        handler = self._handlers[Handlers.PRINT]
        return handler(public_id)

    def download(self, url):
        handler = self._handlers[Handlers.DOWNLOAD]
        return handler(url)



from dcard.base_classes import ResponseExt, HandlerExt


class Response(ResponseExt):
    def format_data(self, data):
        return super().format_data(data)['cities']


class Handler(HandlerExt):
    endpoint = '/city'

    def __call__(self):
        url = self.get_url()
        response = self.get(url)
        return Response(response).data



class BaseAPIError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class CodeError(BaseAPIError):
    code = None

    def __init__(self, errors):
        self.errors = errors
        super().__init__(f'code: {self.code}, ' + str(errors))


class AuthorizationError(BaseAPIError):
    code = 100


class MethodNotExistError(BaseAPIError):
    code = 101


class ClientError(CodeError):
    code = 102


class EAISTOError(CodeError):
    code = 103


class ActionNotAllowedError(CodeError):
    code = 202


class UnknownError(CodeError):
    code = 200


class UnexpectedError(CodeError):
    def __init__(self, code, errors):
        self.code = code
        super().__init__(errors)


class MultiError(BaseAPIError):
    def __init__(self, error_list):
        self.error_list = error_list
        code_list = (error.code for error in self.error_list)
        super().__init__('Error codes: {}'.format(', '.join(map(str, code_list))))



from abc import ABCMeta, abstractmethod

from dcard.extceptions import ClientError, EAISTOError, MultiError, AuthorizationError, MethodNotExistError, \
    UnexpectedError, UnknownError, ActionNotAllowedError

ERROR_MAP = {
    klass.code: klass
    for klass in
    (AuthorizationError, MethodNotExistError, ClientError, EAISTOError, UnknownError, ActionNotAllowedError)
}


class Response(metaclass=ABCMeta):
    _data = None

    @property
    def data(self) -> dict:
        assert self._data is not None, (
            'You must define `._data`'
        )

        return self._data


class Handler(metaclass=ABCMeta):
    @abstractmethod
    def __call__(self, *args, **kwargs) -> Response:
        pass


def get_error(error):
    code, messages = error['code'], error['messages']
    exception_klass = ERROR_MAP.get(code)
    if exception_klass is None:
        return UnexpectedError(code, messages)

    return exception_klass(messages)


class ResponseExt(Response):
    def __init__(self, response):
        content = response.json()
        self._data = self.get_payload(content)

    def get_payload(self, data):
        self.handle_data(data)
        return self.format_data(data)

    def handle_data(self, data):
        errors = data.get('errors', [])
        if errors:
            self.handle_errors(errors)

    def handle_errors(self, errors):
        if len(errors) == 1:
            raise get_error(errors[0])

        raise MultiError(list(map(get_error, errors)))

    def format_data(self, data):
        return {key: value for key, value in data.items() if key != 'errors'}


class HandlerExt(Handler, metaclass=ABCMeta):
    endpoint = None

    def __init__(self, config, request_strategy):
        self.config = config
        self.req_strategy = request_strategy

    @abstractmethod
    def __call__(self, *args, **kwargs):
        pass

    def get_url(self, **kwargs):
        assert self.endpoint is not None
        return self.config['BASE_URL'] + self.endpoint.format(**kwargs)

    def default_query_params(self):
        return {
            'api_key': self.config['API_KEY']
        }

    def get(self, url, params=None):
        result_params = self.default_query_params()
        if params is not None:
            result_params.update(params)

        return self.req_strategy.get(url, params=result_params)

    def post(self, url, json=None):
        params = self.default_query_params()
        return self.req_strategy.post(url, json=json, params=params)



import setuptools

requirements = [
    'schema==0.7.0',
    'requests==2.22.0',
]

setuptools.setup(
    name="dcard_sdk",
    license='MIT',
    # download_url='https://github.com/kirillismad/dcard_wrapper/archive/0.1.2.tar.gz',
    version="0.1.3",
    author="Pech Kirill",
    author_email="pechkirill@gmail.com",
    install_requires=requirements,
    packages=setuptools.find_packages(),
    python_requires='>=3.7',
)



from .create_card_request import Handler as CreateCardRequest
from .list_cities import Handler as ListCities
from .list_tires import Handler as ListTires
from .push_card import Handler as PushCard
from .retrieve_card_request import Handler as RetrieveCardRequest
from .update_card import Handler as UpdateCard
from .deny_card import Handler as DenyCard
from .print_card import Handler as Print
from .download import Handler as Download
